NeedjaTracker = LibStub("AceAddon-3.0"):NewAddon("NeedjaTracker", "AceConsole-3.0", "AceEvent-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTracker", true);

NeedjaTracker.Version = ("$Revision$"):match("%d+") or 0;

NeedjaTracker.MatchChat = {
    -- enUS
    { "(.+) Roll - (.+) for (.+) by (.+)",                                              "RollResult",       { "choice", "roll", "item", "player" } },
    { "You have selected (.+) for: (.+)",                                               "RollRollYou",      { "choice", "item" } },
    { "(.+) has selected (.+) for: (.+)",                                               "RollRollParty",    { "player", "choice", "item" } },
    { "You passed on: (.+)",                                                            "RollPassYou",      { "item" } },
    { "(.+) passed on: (.+)",                                                           "RollPassParty",    { "player", "item" } },
    { "You automatically passed on: (.+) because you cannot loot that item%.",          "RollPassYou",      { "item" } },
    { "(.+) automatically passed on: (.+) because s?he cannot loot that item%.",        "RollPassParty",    { "player", "item" } },
    { "You won: (.+)",                                                                  "RollWinYou",       { "item" } },
    { "(.+) won: (.+)",                                                                 "RollWinParty",     { "player", "item" } },
    { "Everyone passed on: (.+)",                                                       "RollPassAll",      { "item" } },

    -- ruRU
    { "Результат броска (.+) %(\"(.+)\"%) за предмет (.+): (.+)%.",                     "RollResult",       { "player", "choice", "item", "roll" } },
    { "Разыгрывается: (.+)%. Вы сказали: \"(.+)\"%.",                                   "RollRollYou",      { "item", "choice" } },
    { "Разыгрывается: (.+)%. (.+): \"(.+)\"%.",                                         "RollRollParty",    { "item", "player", "choice" } },
    { "Вы отказались от предмета: (.+)",                                                "RollPassYou",      { "item" } },
    { "(.+) отказывается от предмета (.+)%.",                                           "RollPassParty",    { "player", "item" } },
    { "Вы автоматически передаете предмет (.+), поскольку не можете его забрать%.",     "RollPassYou",      { "item" } },
    { "(.+) автоматически передает предмет (.+), поскольку не может его забрать%.",     "RollPassParty",    { "player", "item" } },
    { "Вы пропускаете розыгрыш предмета \"(.+)\", поскольку не можете его забрать%.",   "RollPassYou",      { "item" } },
    { "(.+) пропускает розыгрыш предмета \"(.+)\", поскольку не может его забрать%.",   "RollPassParty",    { "player", "item" } },
    { "Вы выиграли: (.+)",                                                              "RollWinYou",       { "item" } },
    { "(.+) выигрывает: (.+)",                                                          "RollWinParty",     { "player", "item" } },
    { "(.+): предмет никому не нужен%.",                                                "RollPassAll",      { "item" } },
};

NeedjaTracker.MatchNeed = {
    -- enUS
    ["Need"]            = "need",
    ["Greed"]           = "greed",
    ["Disenchant"]      = "disenchant",

    -- ruRU
    ["Нужно"]           = "need",
    ["Мне это нужно"]   = "need",
    ["Не откажусь"]     = "greed",
    ["Распылить"]       = "disenchant",
};

function NeedjaTracker:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("NeedjaTrackerDB");

    self:RegisterChatCommand("needjatracker", "ProcessChatCommand");
    self:RegisterChatCommand("nt", "ProcessChatCommand");

    self:RegisterEvent("CHAT_MSG_LOOT", "ProcessChatMessage");

    self:RegisterEvent("PARTY_MEMBERS_CHANGED", "ProcessPartyEvent");
    self:RegisterEvent("RAID_ROSTER_UPDATE", "ProcessPartyEvent");
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "ProcessPartyEvent");

    ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", function(this, ...) return self:ChatMsgFilter(...) end);
end

function NeedjaTracker:ProcessChatCommand(msg)
    local cmd, next = self:GetArgs(msg, 1);

    if cmd == "stat" then
    elseif cmd == "history" then
    elseif cmd == "clear" then
        self:ClearRollHistory();
    else
        local i;
        for i = 1, L["msg_help_total"] do
            self:PrintToChat(L["msg_help_" .. i]);
        end
    end
end

function NeedjaTracker:ProcessChatMessage(event, msg)
    local n, match;
    for n, match in pairs(self.MatchChat) do
        local p = { msg:match("^" .. match[1] .. "$") };
        if p[1] then
            local args, pos, arg = {};
            for pos, arg in pairs(match[3]) do
                if arg == "player" then
                    args[arg] = self:GetPlayerName(p[pos]);

                elseif arg == "choice" then
                    args[arg] = self.MatchNeed[p[pos]];

                else
                    args[arg] = p[pos];
                end;
            end

            NeedjaTracker[match[2]](self, args);
            break;
        end
    end
end

function NeedjaTracker:ProcessPartyEvent()
end

function NeedjaTracker:ChatMsgFilter(event, msg)
    local n, match;
    for n, match in pairs(self.MatchChat) do
        if msg:match("^" .. match[1] .. "$") then
            return true;
        end
    end
    return false;
end

function NeedjaTracker:RollRollYou(args)
    args.player = UnitName("player");
    self:RollRollParty(args);
end

function NeedjaTracker:RollPassYou(args)
    args.player = UnitName("player");
    self:RollPassParty(args);
end

function NeedjaTracker:RollWinYou(args)
    args.player = UnitName("player");
    self:RollWinParty(args);
end

function NeedjaTracker:RollResult(args)
    local res = self:GetRollResults(args.item, args.player);
    res.choice = args.choice;
    res.roll   = args.roll;

    local stat = self:GetRollStatistics(args.player);
    stat[args.choice] = (stat[args.choice] or 0) + 1;
end

function NeedjaTracker:RollRollParty(args)
    local res = self:GetRollResults(args.item, args.player);
    res.choice = args.choice;
end

function NeedjaTracker:RollPassParty(args)
    local res = self:GetRollResults(args.item, args.player);
    res.choice = "pass";

    local stat = self:GetRollStatistics(args.player);
    stat.pass = (stat.pass or 0) + 1;
end

function NeedjaTracker:RollWinParty(args)
    local res = self:GetRollResults(args.item, args.player);

    local stat = self:GetRollStatistics(args.player);
    stat.win = (stat.win or 0) + 1;

    self:AddRollHistory({ ["item"] = args.item, ["player"] = args.player, ["choice"] = res.choice, ["roll"] = res.roll });
    self:GetRollResults()[args.item] = nil;

    if args.player == UnitName("player") then
        self:PrintToChat((L.msg_win_you):format(args.item, L["roll_" .. (res.choice or 'unknown')], (res.roll or '?')));

    elseif args.choice == "need" then
        self:PrintToChat((L.msg_win_party_need):format(args.item, args.player, L["roll_" .. (res.choice or 'unknown')], (res.roll or '?')));

    else
        self:PrintToChat((L.msg_win_party):format(args.item, args.player, L["roll_" .. (res.choice or 'unknown')], (res.roll or '?')));
    end
end

function NeedjaTracker:RollPassAll(args)
    self:AddRollHistory({ ["item"] = args.item, ["roll"] = "pass" });
    self:GetRollResults()[args.item] = nil;

    self:PrintToChat((L.msg_pass_party):format(args.item));
end

function NeedjaTracker:ClearRollHistory()
    self.db.char.RollHistory = {};
    self.db.char.RollResults = {};
    self.db.char.RollStatistics = {};
end

function NeedjaTracker:AddRollHistory(args)
    local hist = self.db.char.RollHistory;
    if not hist then
        hist = {};
        self.db.char.RollHistory = hist;
    end
    table.insert(hist, args);
    if # hist > 20 then
        table.remove(hist, 1);
    end
end

function NeedjaTracker:GetRollResults(item, player)
    local res = self.db.char.RollResults;
    if not res then
        res = {};
        self.db.char.RollResults = res;
    end
    if not item then
        return res;
    end
    local res_item = res[item];
    if not res_item then
        res_item = {};
        res[item] = res_item;
    end
    if not player then
        return res_item;
    end
    local rep_player = res_item[player];
    if not res_player then
        res_player = {};
        res_item[player] = res_player;
    end
    return res_player;
end

function NeedjaTracker:GetRollStatistics(player)
    local stat = self.db.char.RollStatistics;
    if not stat then
        stat = {};
        self.db.char.RollStatistics = stat;
    end
    if not player then
        return stat;
    end
    local stat_player = stat[player];
    if not stat_player then
        stat_player = {};
        stat[player] = stat_player;
    end
    return stat_player;
end

function NeedjaTracker:GetPlayerName(text)
    local name = text:match("^%|3%-%d+%((.+)%)$");
    if name then
        return name;
    end
    return text;
end

function NeedjaTracker:_TrimColorCodes(text)
    return text
        :gsub("%|c%x%x%x%x%x%x%x%x", "")
        :gsub("%|r", "");
end

function NeedjaTracker:TrimColorCodes(text)
    local filtered = "";
    while text:len() > 0 do
        local p1, p2, p3 = text:match("^(.-)(%|c%x%x%x%x%x%x%x%x%|Hitem:.-%|h%|r)(.*)$");
        if not p1 then
            filtered = filtered .. self:_TrimColorCodes(text);
            break;
        end
        filtered = filtered .. self:_TrimColorCodes(p1) .. p2;
        text = p3;
    end
    return filtered;
end

function NeedjaTracker:PrintToChat(text, channel)
    if channel then
        SendChatMessage(self:TrimColorCodes(text), channel);
    else
        print(text);
    end
end
